unit Generator;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Wave, StdCtrls, MPlayer, ExtCtrls, ComCtrls, Menus, Buttons;

type
  TForm1 = class(TForm)
    FormulaGenBtn: TButton;
    Bevel1: TBevel;
    Image1: TImage;
    StatusBar: TStatusBar;
    FourierBtn: TButton;
    OpenBtn: TButton;
    OpenDialog: TOpenDialog;
    OscBtn: TButton;
    IncTimeBtn: TButton;
    DecTimeBtn: TButton;
    IncScaleBtn: TButton;
    DecScaleBtn: TButton;
    Player: TMediaPlayer;
    SaveBtn: TButton;
    RecFourierBtn: TButton;
    ProcessBtn: TButton;
    PlayBtn: TBitBtn;
    CloseBtn: TBitBtn;
    BasicGenBtn: TButton;
    UserBtn: TButton;
    ComposeBtn: TBitBtn;
    procedure FormulaGenBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FourierBtnClick(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
    procedure PlayBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure OscBtnClick(Sender: TObject);
    procedure IncTimeBtnClick(Sender: TObject);
    procedure DecTimeBtnClick(Sender: TObject);
    procedure IncScaleBtnClick(Sender: TObject);
    procedure DecScaleBtnClick(Sender: TObject);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SaveBtnClick(Sender: TObject);
    procedure RecFourierBtnClick(Sender: TObject);
    procedure ProcessBtnClick(Sender: TObject);
    procedure BasicGenBtnClick(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure UserBtnClick(Sender: TObject);
    procedure ComposeBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  WAV:TWave;
  FileName:String;
  grMode:ShortInt;
  {Processing variables}
  Temp:Array of Single;
  {Fourier variables}
  Int:Array[1..1000] of Single;
  Amp:Array[1..1000,1..2] of Single;
  Frequencies:Array[1..1000] of Single;
  nFrequencies:Word;
  fMax,fMin:Single;
  fCX,fCY:Single;
  {Oscilloscope variables}
  oscTime:Single;
  oscTime1,oscTime2:Single;
  oscSetMarker:Boolean;
  oscCX,oscCY:Single;
  oscDeltaTime:Single;

procedure Fourier;
procedure ShowFourier;
procedure RecoverFourier;
procedure DrawScale;
procedure InitFrequencies;
procedure LinearFilter(const Frequency,FilterDamping:Single);
procedure Echo(const EchoTime,EchoDamping:Single;const Voices:Byte);
procedure WallEcho(const L1,L2:Single);
procedure Noise(const Intensity:Single);
procedure Compression(const Range:Single);
procedure RC_Filter(const t0:Single);
procedure Signal;
procedure ShowWave(const t1,t2:Single);
procedure Progress(const S:Single);
procedure EnableScaling;
procedure DisableScaling;
procedure EnableSaving;
procedure DisableSaving;
procedure ClearProgress;
procedure ClearImage;
procedure Info;
procedure Txt(const S:String);
procedure InitSoundBlaster;

implementation

uses
 Master, Generate, Signal, Compose;

{$R *.dfm}
procedure Progress;
var
 aRect:TRect;
 grs:Integer;
begin
 if S<=1 then
  grs:=round(250*S)
 else
  grs:=250;
 aRect:=Rect(544,4,544 + grs,18);
 Form1.StatusBar.Canvas.Brush.Color:=clRed;
 Form1.StatusBar.Canvas.FillRect(aRect);
end;

procedure ClearProgress;
var
 aRect:TRect;
begin
 aRect:=Rect(544,4,794,18);
 Form1.StatusBar.Canvas.Brush.Color:=clBtnFace;
 Form1.StatusBar.Canvas.FillRect(aRect);
end;

procedure EnableScaling;
begin
 Form1.IncTimeBtn.Enabled:=True;
 Form1.DecTimeBtn.Enabled:=True;
 Form1.IncScaleBtn.Enabled:=True;
 Form1.DecScaleBtn.Enabled:=True;
 oscTime:=0;
 oscDeltaTime:=0.1;
end;

procedure DisableScaling;
begin
 Form1.IncTimeBtn.Enabled:=False;
 Form1.DecTimeBtn.Enabled:=False;
 Form1.IncScaleBtn.Enabled:=False;
 Form1.DecScaleBtn.Enabled:=False;
end;

procedure EnableSaving;
begin
 Form1.SaveBtn.Enabled:=True;
 Form1.BasicGenBtn.Enabled:=True;
 Form1.FormulaGenBtn.Enabled:=True;
 Form1.OpenBtn.Enabled:=True;
 Form1.RecFourierBtn.Enabled:=True;
 Form1.ProcessBtn.Enabled:=True;
 Form1.ComposeBtn.Enabled:=True;
end;

procedure DisableSaving;
begin
 Form1.SaveBtn.Enabled:=False;
 Form1.BasicGenBtn.Enabled:=False;
 Form1.FormulaGenBtn.Enabled:=False;
 Form1.OpenBtn.Enabled:=False;
 Form1.RecFourierBtn.Enabled:=False;
 Form1.ProcessBtn.Enabled:=False;
 Form1.ComposeBtn.Enabled:=False;
end;

procedure ClearImage;
var
 aRect:TRect;
begin
 aRect:=Rect(0,0,750,500);
 Form1.Image1.Canvas.Brush.Color:=clWhite;
 Form1.Image1.Canvas.FillRect(aRect);
end;

procedure Signal;
var
 t,dt:Single;
 x,px:Single;
 y,py:Single;
 z,pz:Single;
 r,sigma:Single;
 px0,py0,pz0:Single;
 a,b,c:Single;
 d,e,f:Single;
 w1,w2,w3,w4:Single;
 k:DWord;
 s:DWord;
begin
 dt:=1000*WAV.DeltaTime;
 WAV.Size:=20*44100;
 t:=0;
 x:=1;  y:=1;  z:=1; a:=10; b:=30; c:=8/3;
 px:=0; py:=0; pz:=0;
 for k:=0 to WAV.Size - 1 do
 begin
  px:=a*(y - x);
  py:=b*x - z*x - y;
  pz:=x*y - c*z;
  x:=x + px*dt;
  y:=y + py*dt;
  z:=z + pz*dt;
  WAV.AudioDataR[k]:=x;
  t:=t + dt;
 end;
 WAV.SetMaxLevel(30000);
end;

procedure Fourier;
var
 k:DWord;
 f:word;
 w:Single;
 t,dt:Single;
 t0,time:Single;
 S1,S2:Single;
 k1,k2:DWord;
begin
 k1:=Round(oscTime1/WAV.DeltaTime);
 k2:=Round(oscTime2/WAV.DeltaTime);
 t0:=oscTime1;
 time:=oscTime2 - oscTime1;
 dt:=WAV.DeltaTime;
 InitFrequencies;
 for f:=1 to nFrequencies do
 begin
  w:=2*pi*Frequencies[f];
  t:=t0;
  S1:=0; S2:=0;
  for k:=k1 to k2 do
  begin
   S1:=S1 + WAV.AudioDataR[k]*cos(w*t)*dt;
   S2:=S2 + WAV.AudioDataR[k]*sin(w*t)*dt;
   t:=t + dt;
   if (k mod 1000)=0 then
    Progress((k - k1)/(k2 - k1));
  end;
  S1:=S1/time;
  S2:=S2/time;
  Amp[f,1]:=S1;
  Amp[f,2]:=S2;
  Int[f]:=sqrt(S1*S1 + S2*S2);
 end;
end;

procedure ShowFourier;{Shows the results of Fourier conversion}
var
 f:Word;
 Max:Single;
 grx,gry:Integer;
begin
 Max:=-100; fMax:=-100; fMin:=1000000;
 for f:=1 to nFrequencies do
  if Int[f]>Max then
   Max:=Int[f];
 for f:=1 to nFrequencies do
 begin
  if Frequencies[f]>fMax then
   fMax:=Frequencies[f];
  if Frequencies[f]<fMin then
   fMin:=Frequencies[f];
 end;
 fCY:=480/Max;
 fCX:=730/(fMax - fMin);
 DrawScale;
 {Drawing the spectrum}
 for f:=1 to nFrequencies do
 begin
  gry:=490 - round(fCY*Int[f]);
  grx:=10 + round(fCX*(Frequencies[f] - fMin));
  Form1.Image1.Canvas.MoveTo(grx,490);
  Form1.Image1.Canvas.LineTo(grx,gry);
 end;
end;

procedure RecoverFourier;
var
 k:DWord;
 f:Word;
 S,S1,S2:Single;
 w,t,dw,dt:Single;
begin
 dt:=WAV.DeltaTime;
 dw:=5;
 t:=0;
 for k:=0 to WAV.Size do
 begin
  S1:=0; S2:=0;
  for f:=1 to nFrequencies do
  begin
   w:=2*pi*Frequencies[f];
   S1:=S1 + Amp[f,1]*cos(w*t)*dw;
   S2:=S2 + Amp[f,2]*sin(w*t)*dw;
  end;
  t:=t + dt;
  S:=S1 + S2;
  WAV.AudioDataR[k]:=S;
  if (k mod 1000)=0 then
   Progress(k/WAV.Size);
 end;
end;

procedure DrawScale;{Draws grid in Fourier window}
var
 i:word;
begin
 {Preparing scale}
 Form1.Image1.Canvas.Pen.Color:=clGreen;
 Form1.Image1.Canvas.MoveTo(5,490);
 Form1.Image1.Canvas.LineTo(745,490);
 Form1.Image1.Canvas.MoveTo(10,10);
 Form1.Image1.Canvas.LineTo(10,495);
 Form1.Image1.Canvas.Pen.Color:=clBlue;
 for i:=1 to 10 do
 begin
  Form1.Image1.Canvas.MoveTo(10 + 73*i,10);
  Form1.Image1.Canvas.LineTo(10 + 73*i,490);
 end;
 for i:=0 to 9 do
 begin
  Form1.Image1.Canvas.MoveTo(10,10 + 48*i);
  Form1.Image1.Canvas.LineTo(740,10 + 48*i);
 end;
 Form1.Image1.Canvas.Pen.Color:=clRed;
end;

procedure ShowWave;
var
 k1,k2:DWord;
 k,dk:DWord;
 grx,gry:Integer;
 lgrx,lgry:Integer;
 S:String;
begin
 ClearImage;
 Form1.Image1.Canvas.Pen.Color:=clBlue;
 Form1.Image1.Canvas.MoveTo(0,250);
 Form1.Image1.Canvas.LineTo(750,250);
 Form1.Image1.Canvas.Pen.Color:=clRed;
 k1:=Round(t1/WAV.DeltaTime);
 if t1<0 then
  k1:=0;
 k2:=Round(t2/WAV.DeltaTime);
 dk:=k2-k1;
 oscCX:=750/dk;
 oscCY:=240/32768;
 lgrx:=0;
 lgry:=250 - Round(oscCY*WAV.AudioDataR[k1]);
 if oscCX>1 then
  for k:=0 to dk do
  begin
   if ((k+k1)>WAV.Size) then
    break;
   grx:=Round(oscCX*k);
   gry:=250 - Round(oscCY*WAV.AudioDataR[k + k1]);
   Form1.Image1.Canvas.MoveTo(lgrx,lgry);
   Form1.Image1.Canvas.LineTo(grx,gry);
   lgrx:=grx; lgry:=gry;
  end;
 if oscCX<1 then
  for grx:=0 to 750 do
  begin
   k:=Round(grx/oscCX);
   if ((k+k1)>WAV.Size) then
    break;
   gry:=250 - Round(oscCY*WAV.AudioDataR[k + k1]);
   Form1.Image1.Canvas.MoveTo(grx,250);
   Form1.Image1.Canvas.LineTo(grx,gry);
  end;
  Str(t1:7:6,S);
  Form1.Image1.Canvas.TextOut(2,252,S);
  Str(t2:7:6,S);
  Form1.Image1.Canvas.TextOut(700,252,S);
end;


procedure InitFrequencies;
var
 f:Word;
begin
 nFrequencies:=200;
 for f:=1 to nFrequencies do
 begin
  Frequencies[f]:=50*f;
 end;
end;

procedure LinearFilter;
var
 k:DWord;
 a:Single;
 x,px:Single;
 t,dt:Single;
 b,c:Single;
begin
 {Initial conditions}
  x:=0; px:=0; t:=0; dt:=WAV.DeltaTime;
  c:=4*pi*pi*Frequency*Frequency; b:=-1*FilterDamping;
 {Processing the sound}
 SetLength(Temp,WAV.Size + 10);
 for k:=0 to WAV.Size do
 begin
  a:=c*(WAV.AudioDataR[k] - x) + b*px;
  px:=px + a*dt;
  x:=x + px*dt;
  Temp[k]:=x;
  t:=t + dt;
  if (k mod 1000)=0 then
   Progress(k/WAV.Size);
 end;
 for k:=1 to WAV.Size do
  WAV.AudioDataR[k]:=Temp[k];
 SetLength(Temp,0);
end;

procedure Echo;
var
 k,d,d1:DWord;
 l:Byte;
 S:Single;
 C:Single;
begin
 SetLength(Temp,WAV.Size + 10);
 d:=Round(EchoTime/WAV.DeltaTime);
 for k:=0 to WAV.Size do
 begin
  S:=WAV.AudioDataR[k];
  C:=EchoDamping;
  for l:=1 to Voices do
  begin
   d1:=l*d;
   if k>d1 then
   begin
    S:=S + C*WAV.AudioDataR[k - d1];
    C:=C*C
   end;
  end;
  Temp[k]:=S;
  if (k mod 1000)=0 then
   Progress(k/WAV.Size);
 end;
 for k:=0 to WAV.Size do
  WAV.AudioDataR[k]:=Temp[k];
 SetLength(Temp,0);
end;

procedure WallEcho;
const
 SoundVelocity:Single=343;
var
 k,d:DWord;
 EchoTime:Single;
 Damping:Single;
begin
 SetLength(Temp,WAV.Size + 10);
 EchoTime:=2*L2/SoundVelocity;
 Damping:=-1*L1*L1/sqr(L1 + 2*L2);
 d:=Round(EchoTime/WAV.DeltaTime);
 for k:=0 to WAV.Size do
 begin
  if k>d then
   Temp[k]:=WAV.AudioDataR[k] + Damping*WAV.AudioDataR[k - d]
  else
   Temp[k]:=WAV.AudioDataR[k];
  if (k mod 1000)=0 then
   Progress(k/WAV.Size);
 end;
 for k:=0 to WAV.Size do
  WAV.AudioDataR[k]:=Temp[k];
 SetLength(Temp,0);
end;


procedure Noise;
var
 k:DWord;
begin
 for k:=0 to WAV.Size do
  WAV.AudioDataR[k]:=WAV.AudioDataR[k] + Intensity*(Random(65536) - 32768)/32768;
end;

procedure Compression;
Var
 k:DWord;
 c:Single;
begin
 WAV.SetMaxLevel(1);
 c:=-1*abs(Range);
 for k:=0 to WAV.Size do
  WAV.AudioDataR[k]:=exp(c*WAV.AudioDataR[k]);
 WAV.SetMaxLevel(30000);
end;

procedure RC_Filter;
var
 k:DWord;
 x,px:Single;
 c,dt:Single;
begin
 SetLength(Temp,WAV.Size + 10);
 dt:=WAV.DeltaTime;
 x:=0; px:=0; c:=1/t0;
 for k:=1 to WAV.Size do
 begin
  px:=WAV.AudioDataR[k] - c*x;
  x:=x + px*dt;
  Temp[k]:=x;
  if (k mod 1000)=0 then
   Progress(k/WAV.Size);
 end;
 for k:=1 to WAV.Size do
  WAV.AudioDataR[k]:=Temp[k];
 SetLength(Temp,0);
end;


procedure TForm1.FormulaGenBtnClick(Sender: TObject);
begin
 GenerateForm.Visible:=True;
end;

procedure Info;
var
 sInfo:String;
begin
 WAV.WaveInfo(sInfo);
 Form1.StatusBar.Panels[0].Text:=sInfo;
end;

procedure Txt;
begin
 Form1.StatusBar.Panels[1].Text:=S;
end;

procedure InitSoundBlaster;
label
 dsploop,check_port,port_not_ready,bad_reset,good_reset;
const
 SBPORT=$1000;
var
 Res:Byte;
begin
 asm
  mov dx, SBPORT + 6
  mov al,1
  out dx,al
  mov cx,40
 dsploop:
  in al,dx
  loop dsploop
  mov al,0
  out dx,al
  add dx,8
  mov cx,100
 check_port:
  in al,dx
  and al,$80
  jz port_not_ready
  sub dx,4
  in al,dx
  add dx,4
  cmp al,$0AA
  je good_reset
 port_not_ready:
  loop check_port
 bad_reset:
  mov byte ptr res,0
  ret
 good_reset:
  mov byte ptr res,1
  ret
 end;
 Form1.FormulaGenBtn.Caption:=IntToStr(res);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 WAV:=TWave.Create(1,44100);
 oscTime:=0;
 oscDeltaTime:=1;
 DisableScaling;
 oscSetMarker:=False;
 oscTime1:=0;
 oscTime2:=100;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 WAV.Free;
end;

procedure TForm1.FourierBtnClick(Sender: TObject);
begin
 grMode:=1;
 DisableScaling;
 ClearProgress;
 Fourier;
 ClearProgress;
 ClearImage;
 ShowFourier;
 Txt('Fourier done');
end;

procedure TForm1.OpenBtnClick(Sender: TObject);
var
 CanOpen:Boolean;
begin
 CanOpen:=OpenDialog.Execute;
 if CanOpen then
 begin
  FileName:=OpenDialog.FileName;
  WAV.OpenWaveFile(FileName);
  ClearProgress;
  Info;
  Txt(FileName + ' opened');
  grMode:=2;
  EnableScaling;
  ShowWave(oscTime,oscTime + oscDeltaTime)
 end;
end;

procedure TForm1.PlayBtnClick(Sender: TObject);
begin
 DisableSaving;
 Player.Open;
 Player.Play;
 CloseBtn.Enabled:=True;
end;

procedure TForm1.CloseBtnClick(Sender: TObject);
begin
 EnableSaving;
 Player.Stop;
 Player.Close;
 CloseBtn.Enabled:=False;
end;

procedure TForm1.OscBtnClick(Sender: TObject);
begin
 grMode:=2;
 EnableScaling;
 ShowWave(oscTime,oscTime + oscDeltaTime);
end;

procedure TForm1.IncTimeBtnClick(Sender: TObject);
begin
 oscTime:=oscTime + oscDeltaTime/3;
 ShowWave(oscTime,oscTime + oscDeltaTime);
end;

procedure TForm1.DecTimeBtnClick(Sender: TObject);
begin
 oscTime:=oscTime - oscDeltaTime/3;
 ShowWave(oscTime,oscTime + oscDeltaTime);
end;

procedure TForm1.IncScaleBtnClick(Sender: TObject);
begin
 oscDeltaTime:=2*oscDeltaTime;
 ShowWave(oscTime,oscTime + oscDeltaTime);
end;

procedure TForm1.DecScaleBtnClick(Sender: TObject);
begin
 oscDeltaTime:=oscDeltaTime/2;
 ShowWave(oscTime,oscTime + oscDeltaTime);
end;

procedure TForm1.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
 grx,gry:Integer;
 Freq,Time:Single;
 Intens,Amp:Single;
 S1,S2:String;
begin
 if grMode=1 then
 begin
  grx:=X;
  gry:=Y;
  Freq:=((grx-10)/fCX) + fMin;
  Intens:=(490 - gry)/fCY;
  Str(Freq:5:0,S1);
  Str(Intens:6:0,S2);
  StatusBar.Panels[1].Text:=S1 + ' Hz. I: ' + S2;
 end;
 if grMode=2 then
 begin
  grx:=X;
  gry:=Y;
  Time:=oscTime + grx*WAV.DeltaTime/oscCX;
  Amp:=(250 - gry)/oscCY;
  Str(Time:7:6,S1);
  Str(Amp:5:0,S2);
  StatusBar.Panels[1].Text:=S1 + ' S. I: ' + S2;
 end;
end;

procedure TForm1.SaveBtnClick(Sender: TObject);
begin
 WAV.SaveWaveFile('Data.wav');
 ClearProgress;
 Txt('Data.wav saved');
end;

procedure TForm1.RecFourierBtnClick(Sender: TObject);
begin
 RecoverFourier;
 grMode:=2;
 EnableScaling;
 ShowWave(oscTime,oscTime + oscDeltaTime);
 ClearProgress;
 Txt('Recovered');
 WAV.SaveWaveFile('Data.wav');
 Txt('Data.wav saved');
 ClearProgress;
end;

procedure TForm1.ProcessBtnClick(Sender: TObject);
begin
 MasterForm.Visible:=True;
end;

procedure TForm1.BasicGenBtnClick(Sender: TObject);
begin
 SignalForm.Visible:=True;
end;

procedure TForm1.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
 Time:Single;
 tempColor:Tcolor;
begin
 Time:=oscTime + X*WAV.DeltaTime/oscCX;
 if oscSetMarker then
 begin
  oscTime2:=Time;
  oscSetMarker:=False
 end
 else
 begin
  oscTime1:=Time;
  oscSetMarker:=True
 end;
 if oscTime1>oscTime2 then
 begin
  Time:=oscTime2;
  oscTime2:=oscTime1;
  oscTime1:=Time
 end;
 StatusBar.Panels[0].Text:=FloatToStr(oscTime1) + '--' + FloatToStr(oscTime2);
 tempColor:=Image1.Canvas.Pen.Color;
 Image1.Canvas.Pen.Color:=clGreen;
 Image1.Canvas.MoveTo(X,0);
 Image1.Canvas.LineTo(X,500);
 Image1.Canvas.Pen.Color:=tempColor;
end;

procedure TForm1.UserBtnClick(Sender: TObject);
begin
 Generator.Signal;
 grMode:=2;
 EnableScaling;
 ShowWave(oscTime,oscTime + oscDeltaTime);
 ClearProgress;
 Txt('Done');
 WAV.SaveWaveFile('Data.wav');
 Txt('Data.wav saved');
 Info;
 ClearProgress;
end;

procedure TForm1.ComposeBtnClick(Sender: TObject);
begin
 ComposeForm.Visible:=True;
end;

end.
