unit Wave;

interface

uses
 SysUtils, Windows;

const
 MaxSize=32*65536;
 WHS=11;{Wave file header size in dwords}

type
 TWave=class(TObject)
  private
   WHeader:array[1..WHS] of DWord;
   LastMaxLevel:Single;
   FSize:DWord;
   procedure FSetSize(const Value:DWord);
   procedure WriteWaveHeader;
   procedure ReadWaveHeader;
  public
   AudioDataR:array of Single;
   AudioDataL:array of Single;
   DeltaTime:Single;
   aTime:Single;
   nChannels:Word;
   nSamplesPerSec:DWord;
   nBitsPerSample:Word;
   property Size:DWord read FSize write FSetSize;
   constructor Create(const Channels:Word; SamplesPerSec:DWord);
   procedure OpenWaveFile(const FileName:String);
   procedure SaveWaveFile(const FileName:String);
   procedure SetMaxLevel(const MaxLevel:Single);
   procedure WaveInfo(var Info:String);
 end;

implementation

uses
 Classes;

var
 Temp:TMemoryStream;
 Data:Array of DWord;

function StrToDWORD(const S:String):DWord;
var
 D,D1:dword;
 ch:char;
 l,i:byte;
begin
 l:=length(S); D1:=1; D:=0;
 for i:=1 to l do
 begin
  ch:=s[i];
  D:=D + D1*ord(ch);
  D1:=D1*256
 end;
 StrToDWORD:=D
end;

constructor TWave.Create;
begin
 inherited Create.nChannels:=Channels;
 inherited Create.nSamplesPerSec:=SamplesPerSec;
 inherited Create.nBitsPerSample:=16;
 inherited Create.DeltaTime:=1/SamplesPerSec;
 inherited Create.Size:=0;
 inherited Create.WriteWaveHeader;
end;

procedure TWave.FSetSize;
begin
 if nChannels=1 then
 begin
  SetLength(AudioDataR,Value + 10);
  SetLength(AudioDataL,0);
 end;
 if nChannels=2 then
 begin
  SetLength(AudioDataR,Value + 10);
  SetLength(AudioDataL,Value + 10);
 end;
 FSize:=Value;
 aTime:=Value*DeltaTime - 0.001;
end;

procedure TWave.WriteWaveHeader;
var
 RSize,D:DWord;
begin
 RSize:=(nBitsPerSample*nChannels*FSize) div 8;
 D:=StrToDWORD('RIFF');
 WHeader[1]:=D;
 WHeader[2]:=RSize + 36;
 D:=StrToDWORD('WAVE');
 WHeader[3]:=D;
 D:=StrToDWORD('fmt ');
 WHeader[4]:=D;
 WHeader[5]:=16;
 WHeader[6]:=65536*nChannels + 1;{ 1 in the first word is the sign of PCM Wave}
 WHeader[7]:=nSamplesPerSec;
 D:=((nSamplesPerSec*nChannels*nBitsPerSample) div 8);
 WHeader[8]:=D;
 WHeader[9]:=65536*nBitsPerSample + ((nChannels*nBitsPerSample) div 8);
 D:=StrToDWORD('data');
 WHeader[10]:=D;
 WHeader[11]:=RSize
end;

procedure TWave.ReadWaveHeader;
begin
 nChannels:=(WHeader[6]-1) div 65536;
 nSamplesPerSec:=WHeader[7];
 nBitsPerSample:=(WHeader[9] div 65536);
end;

procedure TWave.OpenWaveFile;
var
 DataSize:Integer;
 k,i:DWord;
 AR,AL:Word;
 iAR,iAL:SmallInt;
begin
 Temp:=TMemoryStream.Create;
 Temp.LoadFromFile(FileName);
 DataSize:=(Temp.Size div 4);
 Data:=Temp.Memory;
 SetLength(Data,DataSize);
 for k:=0 to 10 do
  WHeader[k+1]:=Data[k];
 ReadWaveHeader;
 if nChannels=2 then
 begin
  Size:=DataSize - 11;
  for k:=11 to DataSize do
  begin
   AR:=Data[k] mod 65536;
   AL:=Data[k] div 65536;
   iAR:=AR; iAL:=AL;
   i:=k-11;
   AudioDataR[i]:=iAR;
   AudioDataL[i]:=iAL;
  end;
 end;
 if nChannels=1 then
 begin
  Size:=2*(DataSize - 11);
  for k:=11 to DataSize do
  begin
   AR:=Data[k] div 65536;
   AL:=Data[k] mod 65536;
   iAR:=AR; iAL:=AL;
   i:=k-11;
   AudioDataR[2*i]:=iAL;
   AudioDataR[2*i + 1]:=iAR;
  end;
 end;
 Temp.Free;
 SetLength(Data,0);
end;

procedure TWave.SaveWaveFile;
var
 k,i:DWord;
 AR,AL:Word;
 iAR,iAL:SmallInt;
begin
 Temp:=TMemoryStream.Create;
 WriteWaveHeader;
 {Stereo file}
 if nChannels=2 then
 begin
  Temp.SetSize(4*Size + 44);
  SetLength(Data,Size + 11);
  Data:=Temp.Memory;
  for k:=0 to 10 do
   Data[k]:=WHeader[k+1];
  for k:=1 to Size do
  begin
   iAR:=round(AudioDataR[k]);
   iAL:=round(AudioDataL[k]);
   AR:=iAR; AL:=iAL;
   i:=k + 10;
   Data[i]:=(AR shl 16) + AL;
  end;
 end;
 {Mono file}
 if nChannels=1 then
 begin
  Temp.SetSize(2*Size + 44);
  SetLength(Data,(Size div 2) + 11);
  Data:=Temp.Memory;
  for k:=0 to 10 do
   Data[k]:=WHeader[k+1];
  for k:=1 to (Size div 2) do
  begin
   iAR:=round(AudioDataR[2*k+1]);
   iAL:=round(AudioDataR[2*k]);
   AR:=iAR; AL:=iAL;
   i:=k + 10;
   Data[i]:=(AR shl 16) + AL;
  end;
 end;
 Temp.SaveToFile('Data.wav');
 Temp.Free;
 SetLength(Data,0);
end;

procedure TWave.SetMaxLevel;
var
 k:DWord;
 AR,AL,cA:Single;
 Max:Single;
begin
 Max:=-1;
 {Mono file}
 if (nChannels=1) then
 begin
  for k:=1 to Size do
  begin
   AR:=abs(AudioDataR[k]);
   if AR>Max then
    Max:=AR
  end;
  cA:=MaxLevel/Max;
  for k:=1 to Size do
   AudioDataR[k]:=cA*AudioDataR[k];
  LastMaxLevel:=MaxLevel
 end;
 {Stereo file}
 if (nChannels=2) then
 begin
  for k:=1 to Size do
  begin
   AR:=abs(AudioDataR[k]);
   AL:=abs(AudioDataL[k]);
   if AR>Max then
    Max:=AR;
   if AL>Max then
    Max:=AL
  end;
  cA:=MaxLevel/Max;
  for k:=1 to Size do
  begin
   AudioDataR[k]:=cA*AudioDataR[k];
   AudioDataL[k]:=cA*AudioDataL[k]
  end;
  LastMaxLevel:=MaxLevel
 end;
end;

procedure TWave.WaveInfo;
var
 Time:single;
 TimeStr:String;
begin
 Info:='';
 Info:=Info + IntToStr(nBitsPerSample) + ' bits ';
 if nChannels=1 then
  Info:=Info + 'Mono ';
 if nChannels=2 then
  Info:=Info + 'Stereo ';
 Info:=Info + IntToStr(nSamplesPerSec) + ' Hz ';
 Info:=Info + IntToStr(2*nChannels*Size) + ' bytes ';
 Time:=Size;
 Time:=Time*DeltaTime;
 Str(Time:4:2,TimeStr);
 Info:=Info + TimeStr + ' Sec';
end;

end.
